// -----------------------------------------------------------------------
// -- Paper Paster.c, a program to convert and install Wallpaper files
// -----------------------------------------------------------------------


// -----------------------------------------------------------------------
// -- #include files


// -----------------------------------------------------------------------
// -- #defines for Paper Paster.c

#define	MASTER_BLOCKS			10

#define	CONVERT_BUTTON			1
#define	INSTALL_BUTTON			11
#define	PREVIEW_ITEM			14

#define	PAT_CREATOR				'RSED'
#define	PAT_TYPE				'rsrc'

#define	WALLPAPER_PATTERN_FILE	'WPat'

#define	DESKTOP_PAT_ID			16


// -----------------------------------------------------------------------
// -- Function prototypes

void	main(void);
void	InitToolbox(void);
Handle	SelectWPFile(void);
void	SavePattern(Handle thePat, Str255 *fName, int vRef);
int		DoInitializations(void);
Handle	ReadDataFork(Str255 *fName, int vRef);
void	InstallDesktopPattern(Handle ppatH);
void	CleanUp(void);
pascal	int		DLOGHook(int theItem, DialogPtr theDialog);
pascal	void	DrawPreview(DialogPtr theDialog, int itemNumber);


// -----------------------------------------------------------------------
// -- External functions needed by Paper Paster.c

extern	Handle			ConvertWallPaper(Handle rawData);
extern	PixPatHandle	ResToPixPat(Handle ppatHandle);


// -----------------------------------------------------------------------
// -- Globals

SFReply			gReply;
SFReply			gLastReply;
PixPatHandle	gCurPat;
Boolean			gDone;
int				gAppResRef;
int				gSysVol;


// -----------------------------------------------------------------------
// -- The main program

void main(void)
{
if (DoInitializations() == noErr)
	{
	while (!gDone)
		{
		SelectWPFile();
		}
	CleanUp();
	}
else
	{
	int		ignore;
	
	InitCursor();
	ignore = Alert(128, nil);
	}
}	// -- Main


// -----------------------------------------------------------------------
// -- Clean up before quitting

void CleanUp(void)
{
PixPatHandle	ppat;
THz				oldZone;

// -- Restore the old desktop pattern

UseResFile(0);
oldZone = GetZone();
SetZone(SystemZone());
ppat = GetPixPat(DESKTOP_PAT_ID);
if (ppat != nil)
	SetDeskCPat(ppat);
UseResFile(gAppResRef);
SetZone(oldZone);
}	// -- CleanUp


// -----------------------------------------------------------------------
// -- Initialize some globals, and make sure we can run on this machine

int DoInitializations(void)
{
int			result = -1;
SysEnvRec	world;

InitToolbox();

gDone = false;
gCurPat = nil;
gAppResRef = CurResFile();

if (SysEnvirons(1, &world) == noErr)
	{
	if (world.hasColorQD)
		result = noErr;
	gSysVol = world.sysVRefNum;
	}
	
return result;
}	// -- DoInitializations


// -----------------------------------------------------------------------
// -- Allow the user to choose a WP file and read its data fork in

Handle SelectWPFile(void)
{
Handle		result = nil;
Point		pt = {-1, -1};
SFTypeList	typeList;
int			err;

typeList[0] = WALLPAPER_PATTERN_FILE;

InitCursor();

// -- Let the user choose a Wallpaper file...

SFGetFile(pt, "\p", nil, 1, &typeList, &DLOGHook, &gReply);
if (gReply.good)
	{
	Handle	h;
	CursHandle		curs;
	
	curs = GetCursor(watchCursor);
	if (curs != nil)
		{
		HLock(curs);
		SetCursor(*curs);
		HUnlock(curs);
		}

	err = SetVol(nil, gReply.vRefNum);

// -- Read the file's data fork into memory

	h = ReadDataFork(&gReply.fName, gReply.vRefNum);
	if (h != nil)
		{
		Handle	ppatH;

// -- Convert the WP file to a standard 'ppat' resource

		ppatH = ConvertWallPaper(h);
		if (ppatH != nil)
			{
			asm
				{
				lea		gReply.fName, A0
				move.l	A0, A1
				clr.l	D0
				move.b	(A0), D0
				add.w	D0, A0
				add.w	#1, A0

				move.l	#'.ppa', (A0)+
				move.b	#'t', (A0)+

				add.b	#5, (A1)
				}
			SavePattern(ppatH, &gReply.fName, gReply.vRefNum);
			}
		}
	}
else
	gDone = true;

return result;
}	// -- SelectWPFile


// -----------------------------------------------------------------------
// -- Our SFGetFile custom dialog procedure

pascal int DLOGHook(int theItem, DialogPtr theDialog)
{
int 	refNum;
OSErr	err;
char	bufPtr[31];
int		wdRefNum;
Boolean	previewUpdate = false;
Rect	r;
Handle	h;
int		k;

// -- See if the user clicked on a different file
	
if ((gReply.vRefNum != gLastReply.vRefNum) ||
	(!EqualString(&gReply.fName, &gLastReply.fName, false, true)))
	{
	long	count;

// -- Copy the reply over

	count = sizeof(SFReply);
	BlockMove(&gReply, &gLastReply, count);
	
// -- See if this is a WP file

	if (gReply.fType == WALLPAPER_PATTERN_FILE)
		{
		err = OpenWD(gReply.vRefNum, CurDirStore, 0, &wdRefNum);
		if (err == noErr)
			{
			Handle	h;

// -- Read the file's data fork into memory

			h = ReadDataFork(&gReply.fName, wdRefNum);
			if (h != nil)
				{
				Handle	ppatH;

// -- Convert the WP file to a standard 'ppat' resource

				ppatH = ConvertWallPaper(h);
				if (ppatH != nil)
					{

// -- Finally, convert the 'ppat' resource to the in-memory format of 'ppat's

					if (gCurPat != nil)
						{
						DisposPixPat(gCurPat);
						gCurPat = nil;
						}
					gCurPat = ResToPixPat(ppatH);
					
					}
				}
			err = CloseWD(wdRefNum);
			}
		}
	else
		{

// -- A WP file isn't selected; no preview!

		if (gCurPat != nil)
			{
			DisposPixPat(gCurPat);
			gCurPat = nil;
			}
		}

// -- A different file was selected, we should update the preview

	previewUpdate = true;
	}

// -- Now handle the item that was clicked on

switch (theItem)
	{

// -- Install a drawing procedure for our UserItem

	case sfHookFirstCall:
		
// -- Set up the default outline user item
		
		GetDItem(theDialog, PREVIEW_ITEM, &k, &h, &r);
		SetDItem(theDialog, PREVIEW_ITEM, k,(Handle)&DrawPreview, &r);
		break;
	
	case INSTALL_BUTTON:
		if ((gReply.fType == WALLPAPER_PATTERN_FILE) && (gCurPat != nil))
			{
			err = OpenWD(gReply.vRefNum, CurDirStore, 0, &wdRefNum);
			if (err == noErr)
				{
				Handle	h;
	
// -- Read the file's data fork into memory
	
				h = ReadDataFork(&gReply.fName, wdRefNum);
				if (h != nil)
					{
					Handle	ppatH;
	
// -- Convert the WP file to a standard 'ppat' resource
	
					ppatH = ConvertWallPaper(h);
					if (ppatH != nil)
						{
						InstallDesktopPattern(ppatH);
						DisposHandle(ppatH);
						}
					}
				err = CloseWD(wdRefNum);
				}
			}
		break;

// -- They clicked on the preview button -- set the desktop pattern

	case PREVIEW_ITEM:
		if ((gReply.fType == WALLPAPER_PATTERN_FILE) && (gCurPat != nil))
			{
			PixPatHandle	pp;
			THz				oldZone;
			
			oldZone = GetZone();
			SetZone(SystemZone());

			pp = NewPixPat();
			CopyPixPat(gCurPat, pp);
			SetDeskCPat(pp);
			SetZone(oldZone);
			}
		break;

	default:
		break;
	}

// -- Handle the proper enabling of buttons

GetDItem(theDialog, INSTALL_BUTTON, &k, &h, &r);

if (gReply.fType == WALLPAPER_PATTERN_FILE)
	{
	ControlHandle	ctl = (ControlHandle)h;
	
	if ((**ctl).contrlHilite != 0)
		HiliteControl(ctl, 0);
	}
else
	{
	ControlHandle	ctl = (ControlHandle)h;

	if ((**ctl).contrlHilite != 255)
		HiliteControl(ctl, 255);
	}
	
//-- Update the preview if we need to

if (previewUpdate)
	{
	GrafPtr	oldPort;
	
	GetPort(&oldPort);
	SetPort(theDialog);
	ClipRect(&theDialog->portRect);
	DrawPreview(theDialog, PREVIEW_ITEM);
	SetPort(oldPort);
	}
	
return theItem;
}	// -- DLOGHook


// -----------------------------------------------------------------------
// -- Install the given 'ppat' handle into the System file

void InstallDesktopPattern(Handle ppatH)
{
PixPatHandle	ppat;
THz				oldZone;
Handle			sysH;
long			sz;
int				mErr;
CursHandle		curs;

curs = GetCursor(watchCursor);
if (curs != nil)
	{
	HLock(curs);
	SetCursor(*curs);
	HUnlock(curs);
	}
	
// -- Load the desktop pattern from the System file

UseResFile(0);
sysH = GetResource('ppat', DESKTOP_PAT_ID);
if (sysH != nil)
	{
	oldZone = GetZone();
	SetZone(HandleZone(sysH));
	HNoPurge(sysH);
	SetZone(oldZone);
	
	sz = GetHandleSize(ppatH);

	oldZone = GetZone();
	SetZone(HandleZone(sysH));
	SetHandleSize(sysH, sz);
	mErr = MemError();
	SetZone(oldZone);
	
	if (mErr == noErr)
		{
		int				err;
		PixPatHandle	ppat;
		
		BlockMove(*ppatH, *sysH, sz);
		ChangedResource(sysH);
		UpdateResFile(0);
		err = FlushVol(nil, gSysVol);
		UseResFile(gAppResRef);
		
// -- Now update the screen
		
		UseResFile(0);
		oldZone = GetZone();
		SetZone(SystemZone());
		ppat = GetPixPat(DESKTOP_PAT_ID);
		if (ppat != nil)
			SetDeskCPat(ppat);
		UseResFile(gAppResRef);
		SetZone(oldZone);
		}
	}

UseResFile(gAppResRef);
InitCursor();
}	// -- InstallDesktopPattern


// -----------------------------------------------------------------------
// -- Read the data fork of the given file into a handle

Handle ReadDataFork(Str255 *fName, int vRef)
{
Handle	result = nil;
int		err;
int		refNum;

// -- Now open up the file's data fork
	
err = FSOpen(fName, vRef, &refNum);
if (err == noErr)
	{
	long	fileSize;
	
// -- Read the file's data fork into a handle

	err = GetEOF(refNum, &fileSize);
	if (err == noErr)
		{
		result = NewHandle(fileSize);
		if (result != nil)
			{
			HLock(result);
			err = FSRead(refNum, &fileSize, (*result));
			HUnlock(result);
			}
		}
	err = FSClose(refNum);
	}

return result;
}	// -- ReadDataFork


// -----------------------------------------------------------------------
// -- Allow the user to save the translated 'ppat'

void SavePattern(Handle thePat, Str255 *fName, int vRef)
{
int			err;

InitCursor();

// -- Delete the file (if it exists) and create it

err = FSDelete(fName, vRef);
err = Create(fName, vRef, PAT_CREATOR, PAT_TYPE);
if (err == noErr)
	{
	int	resRef;

// -- Create a resource fork and open it up

	CreateResFile(fName);
	
	resRef = OpenResFile(fName);
	if (resRef > 0)
		{

// -- Add the resource to the file and close it up

		UseResFile(resRef);
		AddResource(thePat, 'ppat', 128, "\p");
		UpdateResFile(resRef);
		
		DetachResource(thePat);
		CloseResFile(resRef);
		err = FlushVol(nil, vRef);
		}
	}
}	// -- SavePattern


// -------------------------------------------------------------------
// -- Draw the pattern preview button outline

pascal void DrawPreview(DialogPtr theDialog, int itemNumber)
{
Rect	theRect;
int		theType;
Handle	theHandle;

GetDItem(theDialog, itemNumber, &theType, &theHandle, &theRect);
PenSize(1, 1);
PenNormal();

if (gCurPat == nil)
	{

// -- Just draw a gray frame

	PenPat(&gray);
	FrameRect(&theRect);
	InsetRect(&theRect, 1, 1);
	EraseRect(&theRect);
	}
else
	{
	int	oldX, oldY;
	
// -- Draw a square swatch of the pattern

	FrameRect(&theRect);
	InsetRect(&theRect, 1, 1);
	
	oldX = theDialog->portRect.left;
	oldY = theDialog->portRect.top;
	
	SetOrigin(-theRect.left + oldX, -theRect.top + oldY);
	OffsetRect(&theRect, -theRect.left, -theRect.top);
	FillCRect(&theRect, gCurPat);
	SetOrigin(oldX, oldY);
	}
PenNormal();
}	// -- DrawPreview


// -----------------------------------------------------------------------
// -- Initialize the Mac toolbox managers

void InitToolbox(void)
{
int		index;

MaxApplZone();
for (index = 0; index < MASTER_BLOCKS; index++)
	MoreMasters();
InitGraf(&thePort);
InitFonts();
InitWindows();
InitMenus();
TEInit();
InitDialogs(nil);
FlushEvents(everyEvent, 0);
DrawMenuBar();
InitCursor();

// -- Initialize the random number seed to something unique

randSeed = (Ticks + Time);

}	// -- InitToolbox
